///
///    This file is part of ILNumerics Community Edition.
///
///    ILNumerics Community Edition - high performance computing for applications.
///    Copyright (C) 2006 - 2012 Haymo Kutschbach, http://ilnumerics.net
///
///    ILNumerics Community Edition is free software: you can redistribute it and/or modify
///    it under the terms of the GNU General Public License version 3 as published by
///    the Free Software Foundation.
///
///    ILNumerics Community Edition is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
///    GNU General Public License for more details.
///
///    You should have received a copy of the GNU General Public License
///    along with ILNumerics Community Edition. See the file License.txt in the root
///    of your distribution package. If not, see <http://www.gnu.org/licenses/>.
///
///    In addition this software uses the following components and/or licenses: 
///
///    =================================================================================
///    The Open Toolkit Library License
///    
///    Copyright (c) 2006 - 2009 the Open Toolkit library.
///    
///    Permission is hereby granted, free of charge, to any person obtaining a copy
///    of this software and associated documentation files (the "Software"), to deal
///    in the Software without restriction, including without limitation the rights to 
///    use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
///    the Software, and to permit persons to whom the Software is furnished to do
///    so, subject to the following conditions:
///
///    The above copyright notice and this permission notice shall be included in all
///    copies or substantial portions of the Software.
///
///    =================================================================================
///    

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing; 
using ILNumerics.Drawing; 
using ILNumerics.Drawing.Labeling; 

namespace ILNumerics.Drawing.Marker {
    public abstract class ILStyledMarkerShape : ILMarkerShape {

        #region attributes 
        protected MarkerStyle m_style; 
        #endregion

        #region properties
        /// <summary>
        /// The style of markers 
        /// </summary>
        public MarkerStyle Style {
            get {
                return m_style; 
            }
            set {
                m_style = value; 
                CacheMarkerBitmap(); 
            }
        }
        #endregion

        #region constructor
        /// <summary>
        /// create new styled marker
        /// </summary>
        internal ILStyledMarkerShape (ILPanel panel, MarkerStyle style) : base(panel) {
            m_style = style; 
        }
        #endregion

        #region public interface
        public override string ToString() {
            return String.Format(""); 
        }
        public string Hash() {
            string key = Enum.GetName(typeof(MarkerStyle),m_style);
            key = key.Substring(0, 3) + key.Substring(key.Length - 3); 
            return String.Format("M:{0}",key); 
        }
        /// <summary>
        /// convert marker style into bitmap
        /// </summary>
        /// <param name="marker">marker with style property</param>
        /// <returns>bitmap according to style</returns>
        /// <remarks>the bitmap will be loaded from ILNumerics.Drawing ressources. For marker style 'bitmap'
        /// the bitmap referenced in the marker will be returned.</remarks>
        public static Bitmap BitmapFromStyle(MarkerStyle style) {
            switch (style) {
                case MarkerStyle.Circle:
                    return Resources.Markers.Circle25x25;
                case MarkerStyle.Diamond:
                    return Resources.Markers.Diamond25x25;
                case MarkerStyle.TriangleUp:
                    return Resources.Markers.TriangleUp25x25;
                case MarkerStyle.TriangleDown:
                    return Resources.Markers.TriangleDown25x25;
                case MarkerStyle.TriangleLeft:
                    return Resources.Markers.TriangleLeft25x25;
                case MarkerStyle.TriangleRight:
                    return Resources.Markers.TriangleRight25x25;
                case MarkerStyle.Plus:
                    return Resources.Markers.Plus25x25;
                case MarkerStyle.Cross:
                    return Resources.Markers.Cross25x25;
                default:
                    throw new NotImplementedException("ILStyledMarker: marker style not supported!");
            }
        }


        #endregion

        #region private helper

        protected void CacheMarkerBitmap() {
            ILTextureManager mana = m_panel.TextureManager;
            string key = Hash();
            Bitmap markerbmp = BitmapFromStyle(m_style);
            // we need it in format Argb!
            mana.StoreTextureItem(key, markerbmp);
        }
        
        #endregion
    }
}
